"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var _ = require("underscore");
var classHelpers_1 = require("../../../utilities/classHelpers");
var hooks_1 = require("../../../utilities/hooks");
var formCore_1 = require("../formCore");
var validationWrapper_1 = require("../validationWrapper");
var icon_1 = require("./../../display/icon");
function makeComparison(value) {
    return value ? value.trim().toLowerCase() : "";
}
var TagInputRef = function (props, ref) {
    var value = props.value, validationMode = props.validationMode, className = props.className, onChange = props.onChange;
    var _a = React.useState(-1), suggestionIndex = _a[0], setSuggestionIndex = _a[1];
    var _b = React.useState(value || []), tags = _b[0], setTags = _b[1];
    var _c = React.useState([]), suggestions = _c[0], setSuggestions = _c[1];
    var input = React.useRef(undefined);
    var refCallback = React.useCallback(function () {
        return {
            focus: function () {
                if (input.current) {
                    input.current.focus();
                }
            },
            blur: function () {
                if (input.current) {
                    input.current.blur();
                }
            },
            select: function () {
                if (input.current) {
                    input.current.select();
                }
            },
        };
    }, [input]);
    React.useImperativeHandle(ref, refCallback, [refCallback]);
    hooks_1.useDidUpdateEffect(function () {
        setTags(value || []);
    }, value);
    var filterSuggestions = React.useCallback(function (newValue) {
        newValue = makeComparison(newValue);
        if (!newValue) {
            return [];
        }
        var filteredSuggestions = props.suggestions.filter(function (s) { return makeComparison(s).lastIndexOf(newValue, 0) === 0; });
        return filteredSuggestions.filter(function (s) { return tags.indexOf(s) === -1; });
    }, [props.suggestions]);
    var notifyTagsChange = React.useCallback(function (newTags) {
        setTags(newTags);
        setSuggestions([]);
        onChange(newTags);
    }, [onChange]);
    var notifySuggestionsChange = React.useCallback(function (newSuggestions) {
        if (newSuggestions === void 0) { newSuggestions = []; }
        setSuggestions(newSuggestions);
        setSuggestionIndex(-1);
    }, []);
    var onKeyUp = React.useCallback(function (e) {
        var target = e.target;
        // tslint:disable-next-line:no-string-literal
        var targetValue = target["value"];
        switch (e.keyCode) {
            case 40: // ArrowDown
            case 38: // ArrowUp}
                return;
            case 13:
                if (targetValue) {
                    if (tags.indexOf(targetValue) === -1) {
                        var newTags = tags.concat([targetValue]);
                        notifyTagsChange(newTags);
                    }
                    // tslint:disable-next-line:no-string-literal
                    target["value"] = "";
                    return;
                }
                break;
        }
        if (targetValue) {
            notifySuggestionsChange(filterSuggestions(targetValue));
        }
        else {
            notifySuggestionsChange();
        }
    }, [tags, filterSuggestions, notifySuggestionsChange]);
    var onKeyDown = React.useCallback(function (e) {
        var target = e.target;
        // tslint:disable-next-line:no-string-literal
        var targetValue = target["value"];
        switch (e.keyCode) {
            case 8: // delete
                if (tags.length !== 0 && !targetValue) {
                    var newTags = _.clone(tags);
                    newTags.splice(-1, 1);
                    notifyTagsChange(newTags);
                }
                break;
            case 40: // ArrowDown
                if (suggestionIndex < suggestions.length - 1) {
                    setSuggestionIndex(suggestionIndex + 1);
                }
                e.preventDefault();
                break;
            case 38: // ArrowUp}
                if (suggestionIndex > -1) {
                    setSuggestionIndex(suggestionIndex - 1);
                }
                e.preventDefault();
                break;
            case 13:
                var suggestion = suggestions[suggestionIndex];
                if (suggestion) {
                    addTag(suggestion);
                }
                break;
        }
    }, [tags, suggestionIndex, suggestions, notifyTagsChange]);
    var addTag = React.useCallback(function (tag) { return function () {
        notifyTagsChange(tags.concat([tag]));
        input.current.value = "";
        input.current.focus();
    }; }, [tags, notifyTagsChange, input]);
    var removeTag = React.useCallback(function (index) { return function () {
        notifyTagsChange(tags.filter(function (__, idx) { return idx !== index; }));
        input.current.focus();
    }; }, [tags, notifyTagsChange, input]);
    var renderSuggestions = React.useCallback(function (newSuggestions) {
        if (!newSuggestions || !newSuggestions.length) {
            return null;
        }
        return React.createElement("div", { className: "suggestions" }, newSuggestions.map(function (s, i) { return React.createElement("div", { key: s, className: classHelpers_1.ClassHelpers.classNames({ selected: i === suggestionIndex }), onClick: addTag(s) }, s); }));
    }, []);
    var validationMessage = formCore_1.DataValidationMessage.get(props);
    var classes = React.useMemo(function () {
        return classHelpers_1.ClassHelpers.classNames("armstrong-input", "tag-input", className, {
            "show-validation": validationMode !== "none" && validationMessage,
        });
    }, [className, validationMode, validationMessage]);
    return (React.createElement("div", { className: classes },
        tags.map(function (t, i) { return (React.createElement("div", { key: t, className: "tag" },
            t,
            React.createElement(icon_1.Icon, { icon: icon_1.Icon.Icomoon.cross, onClick: removeTag(i) }))); }),
        React.createElement("input", { onKeyDown: onKeyDown, ref: input, onKeyUp: onKeyUp, type: "text" }),
        renderSuggestions(suggestions),
        React.createElement(validationWrapper_1.ValidationLabel, { message: validationMessage, mode: validationMode })));
};
exports.TagInput = React.forwardRef(TagInputRef);
