import * as React from "react";
import * as _ from "underscore";
import { ClassHelpers } from "../../../index";
import { IFormInputHTMLProps } from "../form";
import { DataValidationMessage } from "../formCore";
import { ValidationLabel } from "../validationWrapper";

export interface ICodeInputProps extends IFormInputHTMLProps<React.SelectHTMLAttributes<HTMLInputElement>> {
  lengthPerBox: number[];
  onChange?: (value: any) => void;
  value?: string;
  placeholder?: string;
  type?: string;
  numeric?: boolean;
  readonly?: boolean;
}

function calcTabIndex(tabIndex: number | undefined, fieldIndex: number) {
  if (tabIndex === undefined || tabIndex === -1) {
    return tabIndex;
  }
  return tabIndex + fieldIndex;
}

export const CodeInput: React.FC<ICodeInputProps> = props => {

  const { onChange, validationMode, lengthPerBox, numeric, type, className, tabIndex, value, placeholder, readonly } = props;

  const [focusIndex, setFocusIndex] = React.useState<number>(null)

  const uniq = React.useRef(Math.random());
  const storedKey = React.useRef<string>(null);

  const buildValue = React.useCallback(() => {
    let value: string | number = "";

    let current = document.getElementById(`input_${uniq.current}_0`) as HTMLInputElement;

    lengthPerBox.forEach(() => {
      value += current.value;
      current = current.nextSibling as HTMLInputElement;
    });

    if (numeric) {
      value = parseInt(value, 10);
    }
    if (onChange) {
      onChange(value);
    }
  }, [uniq, lengthPerBox, onChange, numeric])

  const focusNext = React.useCallback((e: React.KeyboardEvent<HTMLInputElement>) => {
    let movingBack = false;
    const current = e.target as HTMLInputElement;
    let el: HTMLInputElement;
    let currentVal = current.value.trim();
    const lpb = lengthPerBox[focusIndex];
    let retFromStore = false;
    currentVal = currentVal.slice(0, lpb);
    if (e.keyCode === 8) {
      movingBack = true;
      if (currentVal.length === 0) {
        el = current.previousSibling as HTMLInputElement;
      } else {
        el = current;
      }
    } else {
      el = current.nextSibling as HTMLInputElement;

      if (
        storedKey.current && el && !el.value) {
        el.value = storedKey.current;

        storedKey.current = null;
        retFromStore = true;
      }
    }
    if (currentVal.length < lpb && !movingBack) {
      return;
    }
    if (el) {
      el.focus();
      if (el && el.value && !movingBack && !retFromStore) {
        el.value = "";
      }
    }

    buildValue();
  }, [lengthPerBox, focusIndex, storedKey, buildValue])

  const handlePaste = React.useCallback((e) => {
    const length = _.reduce(lengthPerBox, (memo, num) => memo + num, 0);
    let pasted: string = e.clipboardData.getData("text/plain").replace(/\s/g, "");
    pasted = pasted.substr(0, length);

    if (numeric) {
      const parsed = parseInt(pasted, 10);
      if (isNaN(parsed)) {
        e.preventDefault();
        return false;
      }
    }

    let current = e.target as HTMLInputElement;
    const splitArray = [];

    let currentIndex = 0;

    lengthPerBox.forEach((lpb) => {
      const chunk = pasted.substr(currentIndex, lpb);
      currentIndex += lpb;
      splitArray.push(chunk);
      current.value = chunk;
      current = current.nextSibling as HTMLInputElement;
    });

    buildValue();
  }, [lengthPerBox, buildValue, numeric])

  const handleFocus = React.useCallback((index: number) => () => {
    setFocusIndex(index)
    storedKey.current = null;
  }, [storedKey])

  const keyDown = React.useCallback((e: React.KeyboardEvent<HTMLInputElement>) => {
    // console.log(e.key)
    const lpb = lengthPerBox[focusIndex];
    const selectionLength = window.getSelection().toString().length;
    if (selectionLength === lpb) {
      return;
    }

    if (e.currentTarget.value.length === lpb) {
      if (e.keyCode >= 48 && e.keyCode <= 57) {
        storedKey.current = e.key;
      }
      if (e.keyCode >= 65 && e.keyCode <= 90) {
        storedKey.current = e.key;
      }
    }
  }, [lengthPerBox, focusIndex, storedKey])

  const handleClick = React.useCallback((e: React.MouseEvent<HTMLInputElement>) => {
    if (e.currentTarget.value) {
      e.currentTarget.select();
    }
  }, [])

  React.useEffect(() => {
    if (value) {
      let currentIndex = 0;
      lengthPerBox.forEach((lpb, i) => {
        const input = document.getElementById(`input_${uniq.current}_${i}`) as HTMLInputElement;
        const chunk =
          value.substr(currentIndex, lpb);
        currentIndex += lpb;
        input.value = chunk;
      });
    }
  }, [])

  const validationMessage = DataValidationMessage.get(props);
  const classes = React.useMemo(() => ClassHelpers.classNames("armstrong-input", "code-input",
    className, {
      "show-validation": validationMode !== "none" && validationMessage,
    }), [className, validationMode, validationMessage]);

  return (
    <div className={classes}>
      <div>
        {
          lengthPerBox.map((lpb, i) => (
            <input id={`input_${uniq.current}_${i}`}
              className="code-input-field"
              tabIndex={calcTabIndex(tabIndex, i)}
              key={`cb_${uniq.current}_${i}`}
              type={type || "text"}
              placeholder={placeholder}
              maxLength={lpb}
              readOnly={readonly}
              onClick={handleClick}
              onFocus={handleFocus(i)}
              onKeyUp={focusNext}
              onKeyDown={keyDown}
              onPaste={handlePaste}
              onChange={buildValue}
            />
          ))}
      </div>
      <ValidationLabel message={validationMessage} mode={validationMode} />
    </div>
  );
}

CodeInput.defaultProps = {
  lengthPerBox: [2, 2, 2],
  validationMode: "none",
}
